local L = LibStub( "AceLocale-3.0" ):GetLocale( "UnderHood" )
local UH = UnderHood

local function colorEntry( order, title, group, key )
	local tbl = {}

	tbl.name = title
	tbl.type = "color"
	tbl.order = order
	tbl.get = function() return unpack( UH.db.profile.colors[group][key] ) end
	tbl.set = function( info, r, g, b ) UH.db.profile.colors[group][key] = { r, g, b } end

	return tbl
end

local options = {
	type = "group",
	handler = UnderHood,
	args = {
		general = {
			name = L["General"],
			desc = L["General options."],
			type = "group",
			order = 1,
			args = {
				config = {
					name = L["Configuration mode"],
					desc = L["Toggles configuration mode."],
					type = "toggle",
					order = 1,
					get = function( info ) return UH.db.profile.general.config end,
					set = "Options_SetConfigMode",
				},
				debug = {
					name = L["Debug"],
					desc = L["Toggles debug mode."],
					type = "toggle",
					order = 2,
					get = function( info ) return UH.db.profile.general.debug end,
					set = function( info, val ) UH.db.profile.general.debug = val end,
				},
				alpha = {
					name = L["Alpha settings"],
					type = "group",
					order = 3,
					get = function( info ) return UH.db.profile.alpha[info[#info]] end,
					set = function( info, value ) UH.db.profile.alpha[info[#info]] = value; UH:UpdateAlpha() end,
					args = {
						combat = {
							name = L["Combat"],
							type = "range",
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.01,
						},
						ooc = {
							name = L["Out of Combat"],
							type = "range",
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.01,
						},
						target = {
							name = L["Have target"],
							type = "range",
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.01,
						},
						regen = {
							name = L["Regenerating"],
							type = "range",
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.01,
						},
						casting = {
							name = L["Casting"],
							type = "range",
							min = 0,
							max = 1,
							step = 0.01,
							bigStep = 0.01,
						},
					},
				},
				colors = {
					name = L["Colors"],
					type = "group",
					order = 4,
					childGroups = "tab",
					args = {
						unit = {},
						power = {},
						pet = {},
						health = {},
						cast = {},
					},
				},
				hideBlizzard = {
				    name = L["Hide Blizzard Frames"],
				    type = "group",
				    order = 5,
				    get = function( info ) return UH.db.profile.hideBlizzard[info[#info]] end,
				    set = function( info, value )
				        UH.db.profile.hideBlizzard[info[#info]] = value
				        UH:UpdateBlizzardFrames()
				    end,
				    args = {
						player = {
						    name = L["Player"],
						    type = "toggle",
						    order = 1,
						},
						target = {
						    name = L["Target"],
						    type = "toggle",
						    order = 2,
						},
						party = {
						    name = L["Party"],
						    type = "toggle",
						    order = 3,
						},
						castbar = {
						    name = L["Cast Bar"],
						    type = "toggle",
						    order = 4,
						},
						aura = {
						    name = L["Buffs/Debuffs"],
						    type = "toggle",
						    order = 5,
						},
					},
				},
			},
			plugins = { modules = {}, },
		},
		modules = {
			name = L["Modules"],
			desc = L["Modules configuration."],
			type = "group",
			order = 2,
			args = {},
		},
	},
	plugins = {},
}

function UnderHood:SetupOptions()
	-- Colors
	local tbl = options.args.general.args.colors.args.unit

	tbl.name = L["Unit colors"]
	tbl.type = "group"
	tbl.order = 1
	tbl.args = {}

	tbl.args.unknown = colorEntry( 1, L["Unknown"], "unit", "unknown" )
	tbl.args.hostile = colorEntry( 2, L["Hostile"], "unit", "hostile" )
	tbl.args.neutral = colorEntry( 3, L["Neutral"], "unit", "neutral" )
	tbl.args.friendly = colorEntry( 4, L["Friendly"], "unit", "friendly" )
	tbl.args.civilian = colorEntry( 5, L["Civilian"], "unit", "civilian" )
	tbl.args.tapped = colorEntry( 6, L["Tapped"], "unit", "tapped" )
	tbl.args.dead = colorEntry( 7, L["Dead"], "unit", "dead" )
	tbl.args.disconnected = colorEntry( 8, L["Disconnected"], "unit", "disconnected" )
	tbl.args.inCombat = colorEntry( 9, L["In Combat"], "unit", "inCombat" )

	tbl = options.args.general.args.colors.args.power

	tbl.name = L["Power colors"]
	tbl.type = "group"
	tbl.order = 2
	tbl.args = {}

	tbl.args.rage = colorEntry( 1, L["Rage"], "power", "rage" )
	tbl.args.energy = colorEntry( 2, L["Energy"], "power", "energy" )
	tbl.args.focus = colorEntry( 3, L["Focus(P)"], "power", "focus" )
	tbl.args.mana = colorEntry( 4, L["Mana"], "power", "mana" )

	tbl = options.args.general.args.colors.args.pet

	tbl.name = L["Pet colors"]
	tbl.type = "group"
	tbl.order = 3
	tbl.args = {}

	tbl.args.happy = colorEntry( 1, L["Happy"], "pet", "happy" )
	tbl.args.neutral = colorEntry( 2, L["Neutral(P)"], "pet", "neutral" )
	tbl.args.angry = colorEntry( 3, L["Angry"], "pet", "angry" )

	tbl = options.args.general.args.colors.args.health

	tbl.name = L["Health colors"]
	tbl.type = "group"
	tbl.order = 4
	tbl.args = {}

	tbl.args.min = colorEntry( 1, L["Minimum"], "health", "min" )
	tbl.args.mid = colorEntry( 2, L["Middle"], "health", "mid" )
	tbl.args.max  = colorEntry( 3, L["Maximum"], "health", "max" )

	tbl = options.args.general.args.colors.args.cast

	tbl.name = L["Spellcast colors"]
	tbl.type = "group"
	tbl.order = 5
	tbl.args = {}

	tbl.args.casting = colorEntry( 1, L["Casting"], "cast", "casting" )
	tbl.args.channeling = colorEntry( 2, L["Channeling"], "cast", "channeling" )
	tbl.args.failed  = colorEntry( 3, L["Cast failed"], "cast", "failed" )
	tbl.args.interrupted = colorEntry( 3, L["Cast interrupted"], "cast", "interrupted" )

	-- Appending modules
	
	for k,v in self:IterateModules() do
		local moduleOptions
		
		if v.GetOptions then
			moduleOptions = v:GetOptions()
		end
		
		if moduleOptions then
			options.args.modules.args[v.moduleName] = moduleOptions
		end
	end
	
	-- Adding profiles
	options.args.profiles = LibStub( "AceDBOptions-3.0" ):GetOptionsTable( self.db )
	options.args.profiles.order = 10

	-- Registering
	LibStub( "AceConfig-3.0" ):RegisterOptionsTable( "UnderHood", options )
	self:RegisterChatCommand( "uh", function() LibStub( "AceConfigDialog-3.0" ):Open( "UnderHood" ) end )
end

function UnderHood:ExtendGeneralSettings( module, moduleOptions )
	options.args.general.plugins.modules[module.moduleName] = moduleOptions
	
	self:NotifyOptionsChanged()
end

function UnderHood:Options_SetConfigMode( info, value )
	self.db.profile.general.config = value
	
	for k, v in self:IterateModules() do
		if type( v.SetConfigMode ) == "function" then
			v:SetConfigMode( value )
		end
	end
	
	self:UpdateAlpha()
	
	if self.db.profile.general.config then
		self:Print( L["Configuration mode on."] )
	else
		self:Print( L["Configuration mode off."] )
	end
end

function UnderHood:NotifyOptionsChanged()
	LibStub( "AceConfigRegistry-3.0" ):NotifyChange( "UnderHood" )
end
